@@ -13,6 +13,8 @@ from slither.core.declarations.solidity_variables import (SolidityFunction,
                                                           SolidityVariableComposed)
 from slither.core.expressions import (Identifier, IndexAccess, MemberAccess,
                                       UnaryOperation)
+from slither.core.solidity_types import UserDefinedType
+from slither.core.solidity_types.type import Type
 from slither.core.source_mapping.source_mapping import SourceMapping
 
 from slither.core.variables.state_variable import StateVariable
@@ -22,6 +24,7 @@ logger = logging.getLogger("Function")
 
 ReacheableNode = namedtuple('ReacheableNode', ['node', 'ir'])
 
+
 class ModifierStatements:
 
     def __init__(self, modifier, entry_point, nodes):
@@ -29,7 +32,6 @@ class ModifierStatements:
         self._entry_point = entry_point
         self._nodes = nodes
 
-
     @property
     def modifier(self):
         return self._modifier
@@ -50,14 +52,16 @@ class ModifierStatements:
     def nodes(self, nodes):
         self._nodes = nodes
 
+
 class FunctionType(Enum):
     NORMAL = 0
     CONSTRUCTOR = 1
     FALLBACK = 2
     RECEIVE = 3
-    CONSTRUCTOR_VARIABLES = 10 # Fake function to hold variable declaration statements
+    CONSTRUCTOR_VARIABLES = 10  # Fake function to hold variable declaration statements
     CONSTRUCTOR_CONSTANT_VARIABLES = 11  # Fake function to hold variable declaration statements
 
+
 class Function(ChildContract, ChildInheritance, SourceMapping):
     """
         Function class
@@ -76,7 +80,7 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         self._entry_point = None
         self._nodes = []
         self._variables = {}
-        self._slithir_variables = set() # slithir Temporary and references variables (but not SSA)
+        self._slithir_variables = set()  # slithir Temporary and references variables (but not SSA)
         self._parameters = []
         self._parameters_ssa = []
         self._parameters_src = None
@@ -121,6 +125,7 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         self._all_solidity_variables_read = None
         self._all_state_variables_written = None
         self._all_slithir_variables = None
+        self._all_nodes = None
         self._all_conditional_state_variables_read = None
         self._all_conditional_state_variables_read_with_loop = None
         self._all_conditional_solidity_variables_read = None
@@ -172,7 +177,7 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
             Return the function signature without the return values
         """
         name, parameters, _ = self.signature
-        return name+'('+','.join(parameters)+')'
+        return name + '(' + ','.join(parameters) + ')'
 
     @property
     def canonical_name(self):
@@ -268,6 +273,15 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         """
         return self._function_type == FunctionType.FALLBACK
 
+    @property
+    def is_receive(self):
+        """
+            Determine if the function is the receive function for the contract
+        Returns
+            (bool)
+        """
+        return self._function_type == FunctionType.RECEIVE
+
     # endregion
     ###################################################################################
     ###################################################################################
@@ -350,8 +364,6 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         """
         return self._is_empty
 
-
-
     # endregion
     ###################################################################################
     ###################################################################################
@@ -475,7 +487,8 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
                             included.
         """
         # This is a list of contracts internally, so we convert it to a list of constructor functions.
-        return [c.modifier.constructors_declared for c in self._explicit_base_constructor_calls if c.modifier.constructors_declared]
+        return [c.modifier.constructors_declared for c in self._explicit_base_constructor_calls if
+                c.modifier.constructors_declared]
 
     @property
     def explicit_base_constructor_calls_statements(self):
@@ -486,7 +499,6 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         # This is a list of contracts internally, so we convert it to a list of constructor functions.
         return list(self._explicit_base_constructor_calls)
 
-
     # endregion
     ###################################################################################
     ###################################################################################
@@ -589,7 +601,7 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         """
             list(SolidityFunction): List of Soldity calls
         """
-        return list(self._internal_calls)
+        return list(self._solidity_calls)
 
     @property
     def high_level_calls(self):
@@ -618,7 +630,6 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         """
         return list(self._low_level_calls)
 
-
     @property
     def external_calls_as_expressions(self):
         """
@@ -716,13 +727,22 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
     ###################################################################################
     ###################################################################################
 
+    @staticmethod
+    def _convert_type_for_signature(t: Type):
+        from slither.core.declarations.contract import Contract
+        if isinstance(t, UserDefinedType) and isinstance(t.type, Contract):
+            return "address"
+        return str(t)
+
     @property
     def signature(self):
         """
             (str, list(str), list(str)): Function signature as
             (name, list parameters type, list return values type)
         """
-        return self.name, [str(x.type) for x in self.parameters], [str(x.type) for x in self.returns]
+        return (self.name,
+                [self._convert_type_for_signature(x.type) for x in self.parameters],
+                [self._convert_type_for_signature(x.type) for x in self.returns])
 
     @property
     def signature_str(self):
@@ -731,7 +751,7 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
             Return the function signature as a str (contains the return values)
         """
         name, parameters, returnVars = self.signature
-        return name+'('+','.join(parameters)+') returns('+','.join(returnVars)+')'
+        return name + '(' + ','.join(parameters) + ') returns(' + ','.join(returnVars) + ')'
 
     # endregion
     ###################################################################################
@@ -752,7 +772,6 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         candidates = [candidate for sublist in candidates for candidate in sublist]
         return [f for f in candidates if f.full_name == self.full_name]
 
-
     # endregion
     ###################################################################################
     ###################################################################################
@@ -801,7 +820,7 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
 
             values += f_new_values(f)
 
-            to_explore += [c for c in f.internal_calls if\
+            to_explore += [c for c in f.internal_calls if \
                            isinstance(c, Function) and c not in explored and c not in to_explore]
             to_explore += [c for (_, c) in f.library_calls if
                            isinstance(c, Function) and c not in explored and c not in to_explore]
@@ -833,6 +852,13 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
                 lambda x: x.slithir_variable)
         return self._all_slithir_variables
 
+    def all_nodes(self):
+        """ recursive version of nodes
+        """
+        if self._all_nodes is None:
+            self._all_nodes = self._explore_functions(lambda x: x.nodes)
+        return self._all_nodes
+
     def all_expressions(self):
         """ recursive version of variables_read
         """
@@ -1036,41 +1062,20 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
 
             f.write("}\n")
 
-    def slithir_cfg_to_dot(self, filename):
-        """
-            Export the function to a dot file
-        Args:
-            filename (str)
-        """
-        from slither.core.cfg.node import NodeType
-        content = ''
-        with open(filename, 'w', encoding='utf8') as f:
-            content += 'digraph{\n'
-            for node in self.nodes:
-                label = 'Node Type: {} {}\n'.format(NodeType.str(node.type), node.node_id)
-                if node.expression:
-                    label += '\nEXPRESSION:\n{}\n'.format(node.expression)
-                if node.irs:
-                    label += '\nIRs:\n' + '\n'.join([str(ir) for ir in node.irs])
-                content += '{}[label="{}"];\n'.format(node.node_id, label)
-                for son in node.sons:
-                    content += '{}->{};\n'.format(node.node_id, son.node_id)
-
-            content += "}\n"
-        return content
-
     def dominator_tree_to_dot(self, filename):
         """
             Export the dominator tree of the function to a dot file
         Args:
             filename (str)
         """
+
         def description(node):
-            desc ='{}\n'.format(node)
+            desc = '{}\n'.format(node)
             desc += 'id: {}'.format(node.node_id)
             if node.dominance_frontier:
                 desc += '\ndominance frontier: {}'.format([n.node_id for n in node.dominance_frontier])
             return desc
+
         with open(filename, 'w', encoding='utf8') as f:
             f.write('digraph{\n')
             for node in self.nodes:
@@ -1080,6 +1085,46 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
 
             f.write("}\n")
 
+    def slithir_cfg_to_dot(self, filename):
+        """
+        Export the CFG to a DOT file. The nodes includes the Solidity expressions and the IRs
+        :param filename:
+        :return:
+        """
+        content = self.slithir_cfg_to_dot_str()
+        with open(filename, 'w', encoding='utf8') as f:
+            f.write(content)
+
+    def slithir_cfg_to_dot_str(self) -> str:
+        """
+        Export the CFG to a DOT format. The nodes includes the Solidity expressions and the IRs
+        :return: the DOT content
+        :rtype: str
+        """
+        from slither.core.cfg.node import NodeType
+        content = ''
+        content += 'digraph{\n'
+        for node in self.nodes:
+            label = 'Node Type: {} {}\n'.format(NodeType.str(node.type), node.node_id)
+            if node.expression:
+                label += '\nEXPRESSION:\n{}\n'.format(node.expression)
+            if node.irs:
+                label += '\nIRs:\n' + '\n'.join([str(ir) for ir in node.irs])
+            content += '{}[label="{}"];\n'.format(node.node_id, label)
+            if node.type == NodeType.IF:
+                true_node = node.son_true
+                if true_node:
+                    content += '{}->{}[label="True"];\n'.format(node.node_id, true_node.node_id)
+                false_node = node.son_false
+                if false_node:
+                    content += '{}->{}[label="False"];\n'.format(node.node_id, false_node.node_id)
+            else:
+                for son in node.sons:
+                    content += '{}->{};\n'.format(node.node_id, son.node_id)
+
+        content += "}\n"
+        return content
+
     # endregion
     ###################################################################################
     ###################################################################################
@@ -1163,7 +1208,6 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         args_vars = self.all_solidity_variables_used_as_args()
         return SolidityVariableComposed('msg.sender') in conditional_vars + args_vars
 
-
     # endregion
     ###################################################################################
     ###################################################################################
@@ -1194,22 +1238,22 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         write_var = list(set(write_var))
         # Remove dupplicate if they share the same string representation
         write_var = [next(obj) for i, obj in groupby(sorted(write_var, key=lambda x: str(x)), lambda x: str(x))]
-        self._expression_vars_written =  write_var
+        self._expression_vars_written = write_var
 
         write_var = [x.variables_written for x in self.nodes]
         write_var = [x for x in write_var if x]
         write_var = [item for sublist in write_var for item in sublist]
         write_var = list(set(write_var))
         # Remove dupplicate if they share the same string representation
-        write_var = [next(obj) for i, obj in\
-                    groupby(sorted(write_var, key=lambda x: str(x)), lambda x: str(x))]
+        write_var = [next(obj) for i, obj in \
+                     groupby(sorted(write_var, key=lambda x: str(x)), lambda x: str(x))]
         self._vars_written = write_var
 
         read_var = [x.variables_read_as_expression for x in self.nodes]
         read_var = [x for x in read_var if x]
         read_var = [item for sublist in read_var for item in sublist]
         # Remove dupplicate if they share the same string representation
-        read_var = [next(obj) for i, obj in\
+        read_var = [next(obj) for i, obj in \
                     groupby(sorted(read_var, key=lambda x: str(x)), lambda x: str(x))]
         self._expression_vars_read = read_var
 
@@ -1217,15 +1261,15 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         read_var = [x for x in read_var if x]
         read_var = [item for sublist in read_var for item in sublist]
         # Remove dupplicate if they share the same string representation
-        read_var = [next(obj) for i, obj in\
+        read_var = [next(obj) for i, obj in \
                     groupby(sorted(read_var, key=lambda x: str(x)), lambda x: str(x))]
         self._vars_read = read_var
 
-        self._state_vars_written = [x for x in self.variables_written if\
+        self._state_vars_written = [x for x in self.variables_written if \
                                     isinstance(x, StateVariable)]
-        self._state_vars_read = [x for x in self.variables_read if\
-                                    isinstance(x, (StateVariable))]
-        self._solidity_vars_read = [x for x in self.variables_read if\
+        self._state_vars_read = [x for x in self.variables_read if \
+                                 isinstance(x, (StateVariable))]
+        self._solidity_vars_read = [x for x in self.variables_read if \
                                     isinstance(x, (SolidityVariable))]
 
         self._vars_read_or_written = self._vars_written + self._vars_read
@@ -1372,7 +1416,6 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         for node in self.nodes:
             node.slithir_generation()
 
-
         self._analyze_read_write()
         self._analyze_calls()
 
@@ -1383,7 +1426,8 @@ class Function(ChildContract, ChildInheritance, SourceMapping):
         compute_dominators(self.nodes)
         compute_dominance_frontier(self.nodes)
         transform_slithir_vars_to_ssa(self)
-        add_ssa_ir(self, all_ssa_state_variables_instances)
+        if not self.contract.is_incorrectly_constructed:
+            add_ssa_ir(self, all_ssa_state_variables_instances)
 
     def update_read_write_using_ssa(self):
         for node in self.nodes:
